//
// Channel render functions.
//

void
draw_param_stub(size_t idx, u8 color) {
    if (idx >= 10) {
        return;
    }
    u8 x0 = PARAMS_START_X + (idx % 5) * PARAMS_BOX_OFFSET_X;
    u8 x1 = x0 + PARAMS_BOX_W;
    u8 y0 = PARAMS_START_Y;
    if (idx >= 5) {
        y0 += PARAMS_BOX_OFFSET_Y;
    }
    u8 y1 = y0 + PARAMS_BOX_H;
    for (size_t i = 0; i < PARAMS_BOX_W; i += 2) {
        draw_pixel(x0 + i + 1, y0, color);
        draw_pixel(x0 + i + 1, y1, color);
    }
    for (size_t i = 0; i < PARAMS_BOX_H; i += 2) {
        draw_pixel(x0, y0 + i + 1, color);
        draw_pixel(x1, y0 + i + 1, color);
    }
}

void
draw_channel_sprite(size_t x, size_t y, u8 clr, u8 idx) {
    draw_icn(x,      y,  &ch_btn_sprite[0 + 6 * idx], clr, 0, 0);
    draw_icn(x + 8,  y,  &ch_btn_sprite[2 + 6 * idx], clr, 0, 0);
    draw_icn(x + 16, y,  &ch_btn_sprite[4 + 6 * idx], clr, 0, 0);
}

void
draw_channels(void) {
    // NOTE: Different channel colors can be interesting.
    u8 colors[] = {
        1, 1, 1, 1,
    };
    for (size_t i = 0; i < 4; i++) {
        bool active = false;
        switch (i) {
            case 0: { active = patterns[pattern_selection_loc].ch1.active; } break;
            case 1: { active = patterns[pattern_selection_loc].ch2.active; } break;
            case 2: { active = patterns[pattern_selection_loc].ch3.active; } break;
            case 3: { active = patterns[pattern_selection_loc].ch4.active; } break;
        }
        u8 clr = active ? colors[i] : COL_OFF;
        size_t y = CHAN_START_Y + i * CHAN_OFFSET_Y;
        draw_channel_sprite(CHAN_START_X, y, clr, i);
    }
}

void
draw_channel_cursor(size_t i, u8 clr) {
    size_t offset_x = 0;
    size_t offset_y = CHAN_H + i * CHAN_OFFSET_Y + 1;
    size_t x0 = CHAN_START_X + offset_x;
    size_t x1 = CHAN_START_X + offset_x + CHAN_W;
    size_t y  = CHAN_START_Y + offset_y;
    switch (settings.cursor) {
        case CURSOR_THICK_LINE: {
            draw_line(x0, y, x1, y, clr);
            draw_line(x0, y + 1, x1, y + 1, clr);
        } break;
        default: {
            draw_line(x0, y, x1, y, clr);
        } break;
    }
}

//
// Trigger render functions.
//

void
clear_trigger(size_t i) {
    size_t offset_x = TRIG_OFFSET_X * (i % 8);
    size_t offset_y = i < 8 ? 0 : TRIG_OFFSET_Y;
    size_t x0 = TRIG_START_X + offset_x + 1;
    size_t x1 = TRIG_START_X + offset_x + TRIG_W - 1;
    size_t y0 = TRIG_START_Y + offset_y + 1;
    size_t y1 = TRIG_START_Y + offset_y + TRIG_H - 4;
    draw_filled_rect(x0, y0, x1, y1, COL_BG);
}

void
draw_trigger(size_t chan, size_t i) {
    TriggerNote trig = {0};
    switch (chan) {
        case 0: { trig = patterns[pattern_selection_loc].ch1.notes[i]; } break;
        case 1: { trig = patterns[pattern_selection_loc].ch2.notes[i]; } break;
        case 2: { trig = patterns[pattern_selection_loc].ch3.notes[i]; } break;
        case 3: { trig = patterns[pattern_selection_loc].ch4.notes[i]; } break;
    }
    if (trig.active) {
        size_t offset_x = TRIG_OFFSET_X * (i % 8);
        size_t offset_y = i < 8 ? 0 : TRIG_OFFSET_Y;
        size_t x = TRIG_START_X + offset_x;
        size_t y = TRIG_START_Y + offset_y;
        const u32 *tile = &note_name_sprites[4 * trig.note];
        draw_icn(x, y, &tile[0], COL_FG, 1, 0);
        draw_icn(x + 8, y, &tile[2], COL_FG, 1, 0);
    } else {
        clear_trigger(i);
    }
}

void
draw_trig_cursor(size_t i, u8 clr) {
    size_t offset_x = TRIG_OFFSET_X * (i % 8);
    size_t offset_y = i < 8 ? 2 : 2 + TRIG_OFFSET_Y;
    size_t x0 = TRIG_START_X + offset_x;
    size_t x1 = TRIG_START_X + TRIG_W + offset_x;
    size_t y  = TRIG_START_Y + TRIG_H + offset_y;
    switch (settings.cursor) {
        case CURSOR_THICK_LINE: {
            draw_line(x0, y, x1, y, clr);
            draw_line(x0, y + 1, x1, y + 1, clr);
        } break;
        default: {
            draw_line(x0, y, x1, y, clr);
        } break;
    }
}

void
draw_right_col_cursor(int i, u8 clr) {
    size_t x0 = 0;
    size_t x1 = 0;
    size_t y = 0;
    switch (i) {
        case R_COL_BPM: {
            x0 = BPM_START_X;
            x1 = x0 + R_COL_W;
            y  = BPM_START_Y + BPM_H + 0;
        } break;
        case R_COL_STOP: {
            x0 = STOP_START_X;
            x1 = x0 + PLAY_STOP_W;
            y  = STOP_START_Y + PLAY_STOP_H + 2;
        } break;
        case R_COL_PLAY: {
            x0 = PLAY_START_X;
            x1 = x0 + PLAY_STOP_W;
            y  = PLAY_START_Y + PLAY_STOP_H + 2;
        } break;
        case R_COL_SETTINGS: {
            x0 = SETTINGS_START_X + 2;
            x1 = x0 + R_COL_W - 4;
            y  = SETTINGS_START_Y + PLAY_STOP_H + 2;
        } break;
        case R_COL_BANK_A: {
            x0 = BANK_START_X;
            x1 = x0 + PAT_W;
            y  = BANK_START_Y + PAT_H + 2;
        } break;
        case R_COL_BANK_B: {
            x0 = BANK_START_X;
            x1 = x0 + PAT_W;
            y  = BANK_START_Y + PAT_H + 2 + 1 * PAT_OFFSET_Y;
        } break;
        case R_COL_BANK_C: {
            x0 = BANK_START_X;
            x1 = x0 + PAT_W;
            y  = BANK_START_Y + PAT_H + 2 + 2 * PAT_OFFSET_Y;
        } break;
        case R_COL_BANK_D: {
            x0 = BANK_START_X;
            x1 = x0 + PAT_W;
            y  = BANK_START_Y + PAT_H + 2 + 3 * PAT_OFFSET_Y;
        } break;
    }
    switch (settings.cursor) {
        case CURSOR_THICK_LINE: {
            draw_line(x0, y, x1, y, clr);
            draw_line(x0, y + 1, x1, y + 1, clr);
        } break;
        default: { draw_line(x0, y, x1, y, clr); } break;
    }
}

void
draw_current_step(u8 step, u8 clr) {
    size_t offset_x = TRIG_OFFSET_X * (step % 8);
    size_t offset_y = step < 8 ? 2 : 2 + TRIG_OFFSET_Y;
    size_t x0 = TRIG_START_X + 3 + offset_x;
    size_t x1 = TRIG_START_X - 3 + TRIG_W + offset_x;
    size_t y  = TRIG_START_Y - 5 + TRIG_H + offset_y;
    draw_line(x0, y, x1, y, clr);
}

void
draw_bank_buttons() {
    size_t x = BANK_START_X;
    size_t y = BANK_START_Y;
    txt_drawf_small("BANK", x - 2, y - 10, COL_FG);
    char bank_names[] = {
        'A', 'B', 'C', 'D',
    };
    for (int i = 0; i < 4; i++) {
        int color = COL_OFF;
        if (i == current_bank) {
            color = COL_FG;
        }
        draw_filled_rect(x, y, x + PAT_W, y + PAT_H, COL_BG);
        draw_rect(x, y, x + PAT_W, y + PAT_H, color);
        txt_drawc(bank_names[i], x + 4, y + 1, color);
        y += PAT_OFFSET_Y;
    }
}

void
draw_pattern_buttons() {
    // Clear patterns.
    size_t x = PAT_START_X;
    size_t y = PAT_START_Y;

    draw_filled_rect(x, y, x + PAT_W, y + PAT_H * 8, COL_BG);

    txt_drawf_small("PAT", x, y - 10, COL_FG);
    char pat_names[] = {
        'A', 'B', 'C', 'D',
        'E', 'F', 'G', 'H',
    };
    for (int i = 0; i < 8; i++) {
        int color = COL_OFF;
        if (i == current_pattern) {
            color = COL_FG;
        }
        if (i == next_pattern && current_pattern != next_pattern) {
            color = COL_ACC_0;
        }
        draw_rect(x, y, x + PAT_W, y + PAT_H, color);
        txt_drawc(pat_names[i], x + 4, y + 1, color);
        y += PAT_OFFSET_Y;
    }
}

void
draw_pattern_cursor(size_t i, u8 clr) {
    size_t offset_x = 0;
    size_t offset_y = PAT_H + i * PAT_OFFSET_Y + 2;
    size_t x0 = PAT_START_X + offset_x;
    size_t x1 = PAT_START_X + offset_x + PAT_W;
    size_t y = PAT_START_Y + offset_y;
    switch (settings.cursor) {
        case CURSOR_THICK_LINE: {
            draw_line(x0, y, x1, y, clr);
            draw_line(x0, y + 1, x1, y + 1, clr);
        } break;
        default: {
            draw_line(x0, y, x1, y, clr);
        } break;
    }
}

void
draw_play() {
    size_t x = PLAY_START_X;
    size_t y = PLAY_START_Y;
    draw_filled_rect(x, y, x + PLAY_STOP_W, y + PLAY_STOP_H, COL_BG);
    draw_rect(x, y, x + PLAY_STOP_W, y + PLAY_STOP_H, COL_ACC_2);
    size_t x_btn = x + PLAY_STOP_W / 2 - 1;
    if (play_status == 1) {
        // Pause button
        draw_filled_rect(x_btn - 1, y + 3, x_btn, y + 7, COL_ACC_2);
        draw_filled_rect(x_btn + 2, y + 3, x_btn + 3, y + 7, COL_ACC_2);
    } else {
        // Play button
        x += 1;
        draw_line(x_btn + 0, y + 2, x_btn + 0, y + 8, COL_ACC_2);
        draw_line(x_btn + 1, y + 3, x_btn + 1, y + 7, COL_ACC_2);
        draw_line(x_btn + 2, y + 4, x_btn + 2, y + 6, COL_ACC_2);
        draw_line(x_btn + 3, y + 5, x_btn + 3, y + 5, COL_ACC_2);
    }
}

void
draw_settings() {
    size_t x = SETTINGS_START_X;
    size_t y = SETTINGS_START_Y;
    draw_rect(x + 2, y, x + R_COL_W - 2, y + PLAY_STOP_H, COL_OFF);
    draw_line(x + 6, y + 4, x + 9, y + 4, COL_OFF);
    draw_line(x + 6, y + 6, x + 9, y + 6, COL_OFF);
    draw_line(x + 15, y + 4, x + 18, y + 4, COL_OFF);
    draw_line(x + 15, y + 6, x + 18, y + 6, COL_OFF);
    draw_line(x + 9, y + 5, x + 15, y + 5, COL_OFF);
}

void
draw_stop() {
    size_t x = STOP_START_X;
    size_t y = STOP_START_Y;
    size_t x_btn = x + PLAY_STOP_W / 2 - 2;
    draw_rect(x, y, x + PLAY_STOP_W, y + PLAY_STOP_H, COL_ACC_1);
    draw_filled_rect(x_btn, y + 3, x_btn + 4, y + 7, COL_ACC_1);
    draw_settings();
}

void
draw_bpm() {
    size_t x = BPM_START_X;
    size_t y = BPM_START_Y + 2;

    // Draw bounding box.
    draw_filled_rect(x, y, x + R_COL_W, y + BPM_H - 4, COL_BG);
    draw_rect(x, y, x + R_COL_W, y + BPM_H - 4, COL_FG);
    draw_line(x + 5, y, x + 19, y, COL_BG);
    txt_drawf_small("BPM", x + 5, y - 4, COL_FG);

    // Make sure its horizontally centered if only 2 digits
    int bpm = patterns[pattern_selection_loc].bpm;
    if (bpm >= 100) {
        txt_drawf("%d", x + 3, y + 5, COL_FG, bpm);
    } else {
        txt_drawf("%d", x + 6, y + 5, COL_FG, bpm);
    }
}

void
draw_triggers(void) {
    for (size_t i = 0; i < 16; i++) {
        size_t offset_x = TRIG_OFFSET_X * (i % 8);
        size_t offset_y = i < 8 ? 0 : 0 + TRIG_OFFSET_Y;
        size_t x0 = TRIG_START_X + offset_x;
        size_t x1 = TRIG_START_X + offset_x + TRIG_W;
        size_t y0 = TRIG_START_Y + offset_y;
        size_t y1 = TRIG_START_Y + offset_y + TRIG_H;
        draw_rect(x0, y0, x1, y1, COL_FG);
        clear_trigger(i);
        draw_trigger(channel_selection_loc, i);
    }
}

void
draw_note(u8 note, u8 clr) {
    size_t octave = note / 12;
    size_t value = note % 12;

    size_t base_x = PIANO_START_X + 2;
    size_t base_y0 = PIANO_START_Y + 2;
    size_t base_y1 = PIANO_START_Y - 2;
    size_t x0 = 0;
    size_t y0 = 0;
    size_t x1 = 0;
    size_t y1 = 0;
    switch (value) {
        // White notes.
        case 0:{
            x0 = base_x + octave * 28;
            x1 = x0 + 1;
            y0 = base_y0;
            y1 = base_y1 + PIANO_H;
            draw_filled_rect(x0, y0, x1, y1, clr);
            x0 = base_x + octave * 28 + 2;
            x1 = x0;
            y0 = base_y1 + PIANO_BLACK_H;
            y1 = y0 + PIANO_H - PIANO_BLACK_H;
            draw_filled_rect(x0, y0, x1, y1, clr);
        } break;
        case 2:{
            x0 = base_x + octave * 28 + 5;
            x1 = x0;
            y0 = base_y0;
            y1 = base_y1 + PIANO_BLACK_H;
            draw_filled_rect(x0, y0, x1, y1, clr);
            x0 = base_x + octave * 28 + 4;
            x1 = x0 + 2;
            y0 = base_y1 + PIANO_BLACK_H;
            y1 = y0 + PIANO_H - PIANO_BLACK_H;
            draw_filled_rect(x0, y0, x1, y1, clr);
        } break;
        case 4:{
            x0 = base_x + octave * 28 + 9;
            x1 = x0 + 1;
            y0 = base_y0;
            y1 = base_y1 + PIANO_H;
            draw_filled_rect(x0, y0, x1, y1, clr);
            x0 = base_x + octave * 28 + 8;
            x1 = x0 + 2;
            y0 = base_y1 + PIANO_BLACK_H;
            y1 = y0 + PIANO_H - PIANO_BLACK_H;
            draw_filled_rect(x0, y0, x1, y1, clr);
        } break;
        case 5:{
            x0 = base_x + octave * 28 + 12;
            x1 = x0 + 1;
            y0 = base_y0;
            y1 = base_y1 + PIANO_H;
            draw_filled_rect(x0, y0, x1, y1, clr);
            x0 = base_x + octave * 28 + 14;
            x1 = x0;
            y0 = base_y1 + PIANO_BLACK_H;
            y1 = y0 + PIANO_H - PIANO_BLACK_H;
            draw_filled_rect(x0, y0, x1, y1, clr);
        } break;
        case 7:{
            x0 = base_x + octave * 28 + 17;
            x1 = x0;
            y0 = base_y0;
            y1 = base_y1 + PIANO_H;
            draw_filled_rect(x0, y0, x1, y1, clr);
            x0 = base_x + octave * 28 + 16;
            x1 = x0 + 2;
            y0 = base_y1 + PIANO_BLACK_H;
            y1 = y0 + PIANO_H - PIANO_BLACK_H;
            draw_filled_rect(x0, y0, x1, y1, clr);
        } break;
        case 9:{
            x0 = base_x + octave * 28 + 21;
            x1 = x0;
            y0 = base_y0;
            y1 = base_y1 + PIANO_H;
            draw_filled_rect(x0, y0, x1, y1, clr);
            x0 = base_x + octave * 28 + 20;
            x1 = x0 + 2;
            y0 = base_y1 + PIANO_BLACK_H;
            y1 = y0 + PIANO_H - PIANO_BLACK_H;
            draw_filled_rect(x0, y0, x1, y1, clr);
        } break;
        case 11: {
            x0 = base_x + octave * 28 + 25;
            x1 = x0 + 1;
            y0 = base_y0;
            y1 = base_y1 + PIANO_H;
            draw_filled_rect(x0, y0, x1, y1, clr);
            x0 = base_x + octave * 28 + 24;
            x1 = x0 + 2;
            y0 = base_y1 + PIANO_BLACK_H;
            y1 = y0 + PIANO_H - PIANO_BLACK_H;
            draw_filled_rect(x0, y0, x1, y1, clr);
        } break;
        default: {
            if (clr == COL_FG) {
                clr = COL_BG;
            }
            y0 = base_y0;
            y1 = base_y1 + PIANO_BLACK_H - 2;
            switch (value) {
                case 1: { x0 = base_x + octave * 28 + 3; } break;
                case 3: { x0 = base_x + octave * 28 + 7; } break;
                case 6: { x0 = base_x + octave * 28 + 15; } break;
                case 8: { x0 = base_x + octave * 28 + 19; } break;
                case 10: { x0 = base_x + octave * 28 + 23; } break;
            }
            x1 = x0;
            draw_line(x0, y0, x1, y1, clr);
        } break;
    }
}

void
draw_piano(void) {
    size_t x0 = PIANO_START_X;
    size_t x1 = PIANO_START_X + PIANO_W;
    size_t y0 = PIANO_START_Y;
    size_t y1 = PIANO_START_Y + PIANO_H;
    draw_rect(x0, y0, x1, y1, COL_FG);
    for (size_t i = 0; i < 12 * 6; i++) {
        draw_note(i, COL_FG);
    }
}

void
draw_params_cursor_wave(size_t i, u8 clr) {
    size_t x = PARAMS_START_X + (i % 5) * PARAMS_BOX_OFFSET_X;
    size_t y = PARAMS_START_Y + PARAMS_BOX_H;
    if (i >= 5) {
        y += PARAMS_BOX_OFFSET_Y;
    }
    u8 x0 = x;
    u8 x1 = x + PARAMS_BOX_W;
    u8 y0 = y - 6;
    u8 y1 = y;
    draw_filled_rect(x0 + 1, y0, x1 - 1, y1, clr);
    draw_line(x0, y0 + 1, x0, y1 - 1, clr);
    draw_line(x1, y0 + 1, x1, y1 - 1, clr);
    {
        size_t x = PARAMS_START_X;
        size_t y = PARAMS_START_Y;
        switch (i) {
            case 0: {
                x += 4 + PARAMS_BOX_OFFSET_X * 0;
                y += PARAMS_BOX_H - 7;
                txt_drawf_small("shape", x, y, COL_BG);
            } break;
            case 1: {
                x += 6 + PARAMS_BOX_OFFSET_X * 1;
                y += PARAMS_BOX_H - 7;
                txt_drawf_small("type", x, y, COL_BG);
            } break;
            case 2: {
                x += 4 + PARAMS_BOX_OFFSET_X * 2;
                y += PARAMS_BOX_H - 7;
                txt_drawf_small("shape", x, y, COL_BG);
            } break;
            case 3: {
                x += 6 + PARAMS_BOX_OFFSET_X * 3;
                y += PARAMS_BOX_H - 7;
                txt_drawf_small("type", x, y, COL_BG);
            } break;
            case 4: {
                x += 6 + PARAMS_BOX_OFFSET_X * 4;
                y += PARAMS_BOX_H - 7;
                txt_drawf_small("prob", x, y, COL_BG);
            } break;
            case 5: {
                x += 4 + PARAMS_BOX_OFFSET_X * 0;
                y += PARAMS_BOX_H - 7 + PARAMS_BOX_OFFSET_Y;
                txt_drawf_small("voice", x, y, COL_BG);
            } break;
            case 6: {
                x += 8 + PARAMS_BOX_OFFSET_X * 1;
                y += PARAMS_BOX_H - 7 + PARAMS_BOX_OFFSET_Y;
                txt_drawf_small("vol", x, y, COL_BG);
            } break;
            case 9: {
                x += 8 + PARAMS_BOX_OFFSET_X * 4;
                y += PARAMS_BOX_H - 7 + PARAMS_BOX_OFFSET_Y;
                txt_drawf_small("pan", x, y, COL_BG);
            } break;
            default: break;
        }
    }
}

void
draw_params_cursor_noise(size_t i, u8 clr) {
    size_t x = PARAMS_START_X + (i % 5) * PARAMS_BOX_OFFSET_X;
    size_t y = PARAMS_START_Y + PARAMS_BOX_H;
    if (i >= 5) {
        y += PARAMS_BOX_OFFSET_Y;
    }
    u8 x0 = x;
    u8 x1 = x + PARAMS_BOX_W;
    u8 y0 = y - 6;
    u8 y1 = y;
    draw_filled_rect(x0 + 1, y0, x1 - 1, y1, clr);
    draw_line(x0, y0 + 1, x0, y1 - 1, clr);
    draw_line(x1, y0 + 1, x1, y1 - 1, clr);
    {
        size_t x = PARAMS_START_X;
        size_t y = PARAMS_START_Y;
        switch (i) {
            case 0: {
                x += 6 + PARAMS_BOX_OFFSET_X * 0;
                y += PARAMS_BOX_H - 7;
                txt_drawf_small("mode", x, y, COL_BG);
            } break;
            case 1: {
                x += 8 + PARAMS_BOX_OFFSET_X * 1;
                y += PARAMS_BOX_H - 7;
                txt_drawf_small("vol", x, y, COL_BG);
            } break;
            case 2: {
                x += 6 + PARAMS_BOX_OFFSET_X * 2;
                y += PARAMS_BOX_H - 7;
                txt_drawf_small("time", x, y, COL_BG);
            } break;
            case 3: {
                x += 8 + PARAMS_BOX_OFFSET_X * 3;
                y += PARAMS_BOX_H - 7;
                txt_drawf_small("dir", x, y, COL_BG);
            } break;
            case 4: {
                x += 6 + PARAMS_BOX_OFFSET_X * 4;
                y += PARAMS_BOX_H - 7;
                txt_drawf_small("prob", x, y, COL_BG);
            } break;
            case 9: {
                x += 8 + PARAMS_BOX_OFFSET_X * 4;
                y += PARAMS_BOX_H - 7 + PARAMS_BOX_OFFSET_Y;
                txt_drawf_small("pan", x, y, COL_BG);
            } break;
            default: break;
        }
    }
}

void
draw_params_cursor_square(size_t i, u8 clr) {
    size_t x = PARAMS_START_X + (i % 5) * PARAMS_BOX_OFFSET_X;
    size_t y = PARAMS_START_Y + PARAMS_BOX_H;
    if (i >= 5) {
        y += PARAMS_BOX_OFFSET_Y;
    }
    u8 x0 = x;
    u8 x1 = x + PARAMS_BOX_W;
    u8 y0 = y - 6;
    u8 y1 = y;
    draw_filled_rect(x0 + 1, y0, x1 - 1, y1, clr);
    draw_line(x0, y0 + 1, x0, y1 - 1, clr);
    draw_line(x1, y0 + 1, x1, y1 - 1, clr);
    {
        size_t x = PARAMS_START_X;
        size_t y = PARAMS_START_Y;
        switch (i) {
            case 0: {
                x += 4 + PARAMS_BOX_OFFSET_X * 0;
                y += PARAMS_BOX_H - 7;
                txt_drawf_small("shape", x, y, COL_BG);
            } break;
            case 1: {
                x += 8 + PARAMS_BOX_OFFSET_X * 1;
                y += PARAMS_BOX_H - 7;
                txt_drawf_small("vol", x, y, COL_BG);
            } break;
            case 2: {
                x += 6 + PARAMS_BOX_OFFSET_X * 2;
                y += PARAMS_BOX_H - 7;
                txt_drawf_small("time", x, y, COL_BG);
            } break;
            case 3: {
                x += 8 + PARAMS_BOX_OFFSET_X * 3;
                y += PARAMS_BOX_H - 7;
                txt_drawf_small("dir", x, y, COL_BG);
            } break;
            case 4: {
                x += 6 + PARAMS_BOX_OFFSET_X * 4;
                y += PARAMS_BOX_H - 7;
                txt_drawf_small("prob", x, y, COL_BG);
            } break;
            case 5: {
                x += 4 + PARAMS_BOX_OFFSET_X * 0;
                y += PARAMS_BOX_H - 7 + PARAMS_BOX_OFFSET_Y;
                txt_drawf_small("sweep", x, y, COL_BG);
            } break;
            case 6: {
                x += 6 + PARAMS_BOX_OFFSET_X * 1;
                y += PARAMS_BOX_H - 7 + PARAMS_BOX_OFFSET_Y;
                txt_drawf_small("time", x, y, COL_BG);
            } break;
            case 7: {
                x += 8 + PARAMS_BOX_OFFSET_X * 2;
                y += PARAMS_BOX_H - 7 + PARAMS_BOX_OFFSET_Y;
                txt_drawf_small("dir", x, y, COL_BG);
            } break;
            case 9: {
                x += 8 + PARAMS_BOX_OFFSET_X * 4;
                y += PARAMS_BOX_H - 7 + PARAMS_BOX_OFFSET_Y;
                txt_drawf_small("pan", x, y, COL_BG);
            } break;
            default: break;
        }
    }
}

void
draw_params_cursor(size_t i, u8 clr) {
    switch (channel_selection_loc) {
        case 0: { draw_params_cursor_square(i, clr); } break;
        case 1: { draw_params_cursor_square(i, clr); } break;
        case 2: { draw_params_cursor_wave(i, clr); } break;
        case 3: { draw_params_cursor_noise(i, clr); } break;
    }
}

IWRAM_CODE
void
draw_wave_pattern(u8 *pattern, int x, int y, u8 clr) {
    for (size_t i = 0; i < 16; ++i) {
        u8 byte = pattern[i];
        u8 first = (byte >> 4) & 0xF;
        u8 second = byte & 0xF;
        u8 a = x + i * 4;
        u8 b = y + 16;
        draw_pixel(a, b - first, clr);
        draw_pixel(a + 1, b - first, clr);
        draw_pixel(a + 2, b - second, clr);
        draw_pixel(a + 3, b - second, clr);
    }
}

IWRAM_CODE
void
clear_parameters(void) {
    size_t x0 = PARAMS_START_X;
    size_t y0 = PARAMS_START_Y;
    size_t x1 = PARAMS_START_X + PARAMS_W;
    size_t y1 = PARAMS_START_Y + PARAMS_H;
    draw_filled_rect(x0, y0, x1, y1, COL_BG);
}

void
draw_prob(u8 prob, u8 clr) {
    size_t x = PARAMS_START_X + PARAMS_BOX_OFFSET_X * 4 + 3;
    size_t y = PARAMS_START_Y + 5;
    switch (prob) {
        case PROB_100: {
            txt_drawf("100", x, y, clr);
            txt_drawc('%', x + 18, y, clr);
        } break;
        case PROB_80: {
            txt_drawf("80", x + 3, y, clr);
            txt_drawc('%', x + 15, y, clr);
        } break;
        case PROB_60: {
            txt_drawf("60", x + 3, y, clr);
            txt_drawc('%', x + 15, y, clr);
        } break;
        case PROB_40: {
            txt_drawf("40", x + 3, y, clr);
            txt_drawc('%', x + 15, y, clr);
        } break;
        case PROB_20: {
            txt_drawf("20", x + 3, y, clr);
            txt_drawc('%', x + 15, y, clr);
        } break;
        case PROB_FIRST: { txt_drawf("1st", x + 3, y, clr); } break;
        case PROB_NOT_FIRST: {
            txt_drawf("1st", x + 3, y, clr);
            draw_line(x + 3, y - 1, x + 21, y - 1, clr);
        } break;
        case PROB_ONE_TWO: { txt_drawf("1:2", x + 3, y, clr); } break;
        case PROB_TWO_TWO: { txt_drawf("2:2", x + 3, y, clr); } break;
        case PROB_ONE_THREE: { txt_drawf("1:3", x + 3, y, clr); } break;
        case PROB_TWO_THREE: { txt_drawf("2:3", x + 3, y, clr); } break;
        case PROB_THREE_THREE: { txt_drawf("3:3", x + 3, y, clr); } break;
        case PROB_ONE_FOUR: { txt_drawf("1:4", x + 3, y, clr); } break;
        case PROB_TWO_FOUR: { txt_drawf("2:4", x + 3, y, clr); } break;
        case PROB_THREE_FOUR: { txt_drawf("3:4", x + 3, y, clr); } break;
        case PROB_FOUR_FOUR: { txt_drawf("4:4", x + 3, y, clr); } break;
        default: break;
    }
}

void
draw_panning(s8 pan, u8 clr) {
    switch (settings.sync) {
        case SYNC_OUT_LINK_AUDIO_4:
        case SYNC_OUT_LINK_AUDIO_8:
        case SYNC_OUT_LINK_AUDIO_16:
        case SYNC_OUT_AUDIO_4:
        case SYNC_OUT_AUDIO_8:
        case SYNC_OUT_AUDIO_16: { clr = COL_OFF; } break;
        default: break;
    }
    size_t x = PARAMS_START_X + PARAMS_BOX_OFFSET_X * 4 + 3;
    size_t y = PARAMS_START_Y + PARAMS_BOX_OFFSET_Y + 5;
    switch (pan) {
        case -1: {
            txt_drawf("L", x + 9, y, clr);
        } break;
        case 0: {
            txt_drawf("MID", x + 3, y, clr);
        } break;
        case 1: {
            txt_drawf("R", x + 9, y, clr);
        } break;
        default: break;
    }
}

IWRAM_CODE
void
draw_parameters_wave(ChannelWaveParams *params, bool global) {
    u8 col_fg = COL_FG;
    if (global && input_handler == handle_channel_selection) {
        col_fg = COL_OFF;
    }

    // Draw current wave data.
    {
        size_t x = PARAMS_START_X;
        size_t y = PARAMS_START_Y;

        // Wave Patterns.
        u8 col_wave_a = col_fg != COL_OFF && (params->wave_mode == 0 || params->wave_mode == 2) ? COL_ACC_1 : COL_OFF;
        u8 col_wave_b = col_fg != COL_OFF && (params->wave_mode == 1 || params->wave_mode == 2) ? COL_ACC_2 : COL_OFF;
        draw_wave_pattern(waves[params->shape_a][params->type_a], x, y, col_wave_a);
        draw_wave_pattern(waves[params->shape_b][params->type_b], x + PARAMS_BOX_OFFSET_X * 2, y, col_wave_b);
    }

    // Trig probability.
    draw_prob(params->prob, col_fg);

    // Trig pannning.
    draw_panning(params->pan, col_fg);

    // Mode selection.
    {
        size_t x = PARAMS_START_X;
        size_t y = PARAMS_START_Y + PARAMS_BOX_OFFSET_Y;
        switch (params->wave_mode) {
            case 0: { txt_drawf("A", x + 12, y + 5, col_fg); } break;
            case 1: { txt_drawf("B", x + 12, y + 5, col_fg); } break;
            case 2: { txt_drawf("A+B", x + 6, y + 5, col_fg); } break;
        }
    }

    // Wave volume.
    {
        size_t x = PARAMS_START_X + PARAMS_BOX_OFFSET_X + 3;
        size_t y = PARAMS_START_Y + PARAMS_BOX_OFFSET_Y + 5;
        switch (params->wave_volume) {
            case 0: {
                txt_drawf("0", x + 6, y, col_fg);
                txt_drawc('%', x + 12, y, col_fg);
            } break;
            case 1: {
                txt_drawf("25", x + 3, y, col_fg);
                txt_drawc('%', x + 15, y, col_fg);
            } break;
            case 2: {
                txt_drawf("50", x + 3, y, col_fg);
                txt_drawc('%', x + 15, y, col_fg);
            } break;
            case 3: {
                txt_drawf("75", x + 3, y, col_fg);
                txt_drawc('%', x + 15, y, col_fg);
            } break;
            case 4: {
                txt_drawf("100", x, y, col_fg);
                txt_drawc('%', x + 18, y, col_fg);
            } break;
        }
    }

    // Labels.
    {
        size_t x = PARAMS_START_X;
        size_t y = PARAMS_START_Y;
        txt_drawf_small("shape", x + 4 + PARAMS_BOX_OFFSET_X * 0, y + PARAMS_BOX_H - 7, col_fg);
        txt_drawf_small("type", x + 6 + PARAMS_BOX_OFFSET_X * 1, y + PARAMS_BOX_H - 7, col_fg);
        txt_drawf_small("shape", x + 4 + PARAMS_BOX_OFFSET_X * 2, y + PARAMS_BOX_H - 7, col_fg);
        txt_drawf_small("type", x + 6 + PARAMS_BOX_OFFSET_X * 3, y + PARAMS_BOX_H - 7, col_fg);
        txt_drawf_small("prob", x + 6 + PARAMS_BOX_OFFSET_X * 4, y + PARAMS_BOX_H - 7, col_fg);
        y += PARAMS_BOX_OFFSET_Y;
        txt_drawf_small("voice", x + 4 + PARAMS_BOX_OFFSET_X * 0, y + PARAMS_BOX_H - 7, col_fg);
        txt_drawf_small("vol", x + 8 + PARAMS_BOX_OFFSET_X * 1, y + PARAMS_BOX_H - 7, col_fg);
        txt_drawf_small("pan", x + 8 + PARAMS_BOX_OFFSET_X * 4, y + PARAMS_BOX_H - 7, col_fg);
    }

    // Empty spacers.
    {
        draw_param_stub(7, COL_OFF);
        draw_param_stub(8, COL_OFF);
    }
}

void
draw_parameters_square(ChannelSquareParams *params, bool sweep, bool global) {
    u8 col_fg = COL_FG;
    if (global && input_handler == handle_channel_selection) {
        col_fg = COL_OFF;
    }

    // Duty cycle / shape.
    {
        size_t x = PARAMS_START_X + 3;
        size_t y = PARAMS_START_Y + 1;

        size_t x0 = x + 2;
        size_t x1 = x0;
        size_t x2 = x0;
        size_t x3 = x0;
        size_t x4 = x0;
        size_t x5 = x0;
        size_t y0 = y + 14;
        size_t y1 = y + 2;

        switch (params->duty_cycle) {
            case 0: {
                x1 += 4;
                x2 += 6;
                x3 += 13;
                x4 += 15;
                x5 += 20;
            } break;
            case 1: {
                x1 += 4;
                x2 += 7;
                x3 += 13;
                x4 += 16;
                x5 += 20;
            } break;
            case 2: {
                x1 += 3;
                x2 += 8;
                x3 += 12;
                x4 += 17;
                x5 += 20;
            } break;
            case 3: {
                x1 += 2;
                x2 += 9;
                x3 += 11;
                x4 += 18;
                x5 += 20;
            } break;
        }
        u8 col_shape = col_fg != COL_OFF ? COL_ACC_1 : COL_OFF;
        draw_line(x0, y0, x1, y0, col_shape);
        draw_line(x1, y1, x1, y0, col_shape);
        draw_line(x1, y1, x2, y1, col_shape);
        draw_line(x2, y1, x2, y0, col_shape);
        draw_line(x2, y0, x3, y0, col_shape);
        draw_line(x3, y1, x3, y0, col_shape);
        draw_line(x3, y1, x4, y1, col_shape);
        draw_line(x4, y1, x4, y0, col_shape);
        draw_line(x4, y0, x5, y0, col_shape);
    }

    // Envelope.
    {
        size_t x = PARAMS_START_X + PARAMS_BOX_OFFSET_X * 1 + 1;
        size_t y = PARAMS_START_Y + 1;
        size_t x0 = x;
        size_t y0 = y + 15 - params->env_volume;
        size_t x1 = x + (3 * PARAMS_BOX_OFFSET_X) * params->env_time / 8 + 7;
        size_t y1 = params->env_direction == 0 ? y + 15 : y;
        size_t x2 = x + PARAMS_BOX_OFFSET_X * 3 - 5;
        size_t y2 = y1;

        // Env.
        u8 col_env = col_fg != COL_OFF ? COL_ACC_2 : COL_OFF;
        if (params->env_time == 0) {
            draw_line(x0, y0, x2, y0, col_env);
        } else {
            draw_line(x0, y0, x1, y1, col_env);
            draw_line(x1, y1, x2, y2, col_env);
        }
    }

    // Trig probability.
    draw_prob(params->prob, col_fg);

    // Trig pannning.
    draw_panning(params->pan, col_fg);

    // Sweep.
    if (sweep) {
        size_t x = PARAMS_START_X;
        size_t y = PARAMS_START_Y + PARAMS_BOX_OFFSET_Y;

        // Bounding box.
        u8 x0 = x + 1;
        u8 x1 = x + PARAMS_BOX_OFFSET_X * 3 - 5;
        u8 y0 = y + 2;
        u8 y1 = y + PARAMS_BOX_H - 8;
        draw_line(x0, y0, x1, y0, col_fg);
        draw_line(x0, y1, x1, y1, col_fg);
        draw_line(x0 - 1, y0 + 1, x0 - 1, y1 - 1, col_fg);
        draw_line(x1 + 1, y0 + 1, x1 + 1, y1 - 1, col_fg);

        // Number.
        switch (params->sweep_number) {
            case 0: { txt_drawf("0", x + 12, y + 5, col_fg); } break;
            case 1: { txt_drawf("1", x + 12, y + 5, col_fg); } break;
            case 2: { txt_drawf("2", x + 12, y + 5, col_fg); } break;
            case 3: { txt_drawf("3", x + 12, y + 5, col_fg); } break;
            case 4: { txt_drawf("4", x + 12, y + 5, col_fg); } break;
            case 5: { txt_drawf("5", x + 12, y + 5, col_fg); } break;
            case 6: { txt_drawf("6", x + 12, y + 5, col_fg); } break;
            case 7: { txt_drawf("7", x + 12, y + 5, col_fg); } break;
        }

        // Time.
        x += PARAMS_BOX_OFFSET_X;
        switch (params->sweep_time) {
            case 0: { txt_drawf("Off", x + 6, y + 5, col_fg); } break;
            case 1: { txt_drawf("7ms", x + 6, y + 5, col_fg); } break;
            case 2: { txt_drawf("15ms", x + 3, y + 5, col_fg); } break;
            case 3: { txt_drawf("23ms", x + 3, y + 5, col_fg); } break;
            case 4: { txt_drawf("31ms", x + 3, y + 5, col_fg); } break;
            case 5: { txt_drawf("39ms", x + 3, y + 5, col_fg); } break;
            case 6: { txt_drawf("46ms", x + 3, y + 5, col_fg); } break;
            case 7: { txt_drawf("54ms", x + 3, y + 5, col_fg); } break;
        }

        // Direction.
        x += PARAMS_BOX_OFFSET_X;
        char arr_down[2] = { 0x19, 0 };
        char arr_up[2] = { 0x18, 0 };
        switch (params->sweep_direction) {
            case 0: { txt_drawf(arr_up, x + 12, y + 5, col_fg); } break;
            case 1: { txt_drawf(arr_down, x + 12, y + 5, col_fg); } break;
        }
    }

    // Labels.
    {
        size_t x = PARAMS_START_X;
        size_t y = PARAMS_START_Y;
        txt_drawf_small("shape", x + 4 + PARAMS_BOX_OFFSET_X * 0, y + PARAMS_BOX_H - 7, col_fg);
        txt_drawf_small("vol", x + 8 + PARAMS_BOX_OFFSET_X * 1, y + PARAMS_BOX_H - 7, col_fg);
        txt_drawf_small("time", x + 6 + PARAMS_BOX_OFFSET_X * 2, y + PARAMS_BOX_H - 7, col_fg);
        txt_drawf_small("dir", x + 8 + PARAMS_BOX_OFFSET_X * 3, y + PARAMS_BOX_H - 7, col_fg);
        txt_drawf_small("prob", x + 6 + PARAMS_BOX_OFFSET_X * 4, y + PARAMS_BOX_H - 7, col_fg);
        y += PARAMS_BOX_OFFSET_Y;
        if (sweep) {
            txt_drawf_small("sweep", x + 4 + PARAMS_BOX_OFFSET_X * 0, y + PARAMS_BOX_H - 7, col_fg);
            txt_drawf_small("time", x + 6 + PARAMS_BOX_OFFSET_X * 1, y + PARAMS_BOX_H - 7, col_fg);
            txt_drawf_small("dir", x + 8 + PARAMS_BOX_OFFSET_X * 2, y + PARAMS_BOX_H - 7, col_fg);
        }
        txt_drawf_small("pan", x + 8 + PARAMS_BOX_OFFSET_X * 4, y + PARAMS_BOX_H - 7, col_fg);
    }

    // Empty spacers.
    {
        if (!sweep) {
            draw_param_stub(5, COL_OFF);
            draw_param_stub(6, COL_OFF);
            draw_param_stub(7, COL_OFF);
        }
        draw_param_stub(8, COL_OFF);
    }
}

void
draw_parameters_noise(ChannelNoiseParams* params, bool global) {
    u8 col_fg = COL_FG;
    if (global && input_handler == handle_channel_selection) {
        col_fg = COL_OFF;
    }

    // Mode.
    {
        size_t x = PARAMS_START_X;
        size_t y = PARAMS_START_Y;
        switch (params->bit_mode) {
            case 0: { txt_drawf("A", x + 12, y + 5, col_fg); } break;
            case 1: { txt_drawf("B", x + 12, y + 5, col_fg); } break;
        }
    }

    // Envelope.
    {
        size_t x = PARAMS_START_X + PARAMS_BOX_OFFSET_X * 1 + 1;
        size_t y = PARAMS_START_Y + 1;
        size_t x0 = x;
        size_t y0 = y + 15 - params->env_volume;
        size_t x1 = x + (3 * PARAMS_BOX_OFFSET_X) * params->env_time / 8 + 7;
        size_t y1 = params->env_direction == 0 ? y + 15 : y;
        size_t x2 = x + PARAMS_BOX_OFFSET_X * 3 - 5;
        size_t y2 = y1;

        // Env.
        u8 col_env = col_fg != COL_OFF ? COL_ACC_2 : COL_OFF;
        if (params->env_time == 0) {
            draw_line(x0, y0, x2, y0, col_env);
        } else {
            draw_line(x0, y0, x1, y1, col_env);
            draw_line(x1, y1, x2, y2, col_env);
        }
    }

    // Trig probability.
    draw_prob(params->prob, col_fg);

    // Trig pannning.
    draw_panning(params->pan, col_fg);

    // Labels.
    {
        size_t x = PARAMS_START_X;
        size_t y = PARAMS_START_Y;
        txt_drawf_small("mode", x + 6 + PARAMS_BOX_OFFSET_X * 0, y + PARAMS_BOX_H - 7, col_fg);
        txt_drawf_small("vol", x + 8 + PARAMS_BOX_OFFSET_X * 1, y + PARAMS_BOX_H - 7, col_fg);
        txt_drawf_small("time", x + 6 + PARAMS_BOX_OFFSET_X * 2, y + PARAMS_BOX_H - 7, col_fg);
        txt_drawf_small("dir", x + 8 + PARAMS_BOX_OFFSET_X * 3, y + PARAMS_BOX_H - 7, col_fg);
        txt_drawf_small("prob", x + 6 + PARAMS_BOX_OFFSET_X * 4, y + PARAMS_BOX_H - 7, col_fg);
        y += PARAMS_BOX_OFFSET_Y;
        txt_drawf_small("pan", x + 8 + PARAMS_BOX_OFFSET_X * 4, y + PARAMS_BOX_H - 7, col_fg);
    }

    // Empty spacers.
    {
        draw_param_stub(5, COL_OFF);
        draw_param_stub(6, COL_OFF);
        draw_param_stub(7, COL_OFF);
        draw_param_stub(8, COL_OFF);
    }
}

void
draw_parameters(void) {
    clear_parameters();
    Pattern *pat = &patterns[pattern_selection_loc];
    // DEBUG: Draw the parameter window
    // draw_rect(PARAMS_START_X, PARAMS_START_Y, PARAMS_START_X + PARAMS_W, PARAMS_START_Y + PARAMS_H, COL_OFF);
    // DEBUG: Drawing the reference grid overlay
    // for (size_t i = 0; i < 10; i++) {
    //     draw_param_stub(i, COL_OFF);
    // }
    if (input_handler == handle_trigger_selection ||
            input_handler == handle_param_selection_sq1 ||
            input_handler == handle_param_selection_sq2 ||
            input_handler == handle_param_selection_wave ||
            input_handler == handle_param_selection_noise) {
        switch (channel_selection_loc) {
            case 0: { draw_parameters_square(&pat->ch1.params[trig_selection_loc], true, false); } break;
            case 1: { draw_parameters_square(&pat->ch2.params[trig_selection_loc], false, false); } break;
            case 2: { draw_parameters_wave(&pat->ch3.params[trig_selection_loc], false); } break;
            case 3: { draw_parameters_noise(&pat->ch4.params[trig_selection_loc], false); } break;
        }
        return;
    }
    if (input_handler == handle_channel_selection ||
            input_handler == handle_param_selection_ch1 ||
            input_handler == handle_param_selection_ch2 ||
            input_handler == handle_param_selection_ch3 ||
            input_handler == handle_param_selection_ch4) {
        switch (channel_selection_loc) {
            case 0: { draw_parameters_square(&ch1_params, true, true); } break;
            case 1: { draw_parameters_square(&ch2_params, false, true); } break;
            case 2: { draw_parameters_wave(&ch3_params, true); } break;
            case 3: { draw_parameters_noise(&ch4_params, true); } break;
        }
        return;
    }
}

void
clear_cursors(void) {
    draw_trig_cursor(last_trig_loc, COL_BG);
    draw_channel_cursor(last_channel_loc, COL_BG);
    draw_pattern_cursor(last_pattern_loc, COL_BG);
    draw_right_col_cursor(last_right_col_loc, COL_BG);
    for (size_t i = 0; i < 16; i++) {
        draw_current_step(i, COL_OFF);
    }
}

void
draw_pattern_chain_cursor(void) {
    size_t offset_x = (PAT_TRIG_H + 7) * (param_selection_loc % 8);
    size_t offset_y = param_selection_loc < 8 ? 0 : 0 + PAT_TRIG_OFFSET_Y;
    size_t x0 = PAT_TRIG_START_X + offset_x;
    size_t x1 = PAT_TRIG_START_X + offset_x + PAT_TRIG_W;
    size_t y  = PAT_TRIG_START_Y + offset_y + PAT_TRIG_H + 2;
    switch (settings.cursor) {
        case CURSOR_THICK_LINE: {
            draw_line(x0, y, x1, y, COL_ACC_0);
            draw_line(x0, y + 1, x1, y + 1, COL_ACC_0);
        } break;
        default: {
            draw_line(x0, y, x1, y, COL_ACC_0);
        } break;
    }
}

void
draw_cursors(void) {
    clear_cursors();
    draw_current_step(step_counter, COL_ACC_1);
    if (input_handler == handle_trigger_selection) {
        draw_trig_cursor(trig_selection_loc, COL_ACC_0);
    }
    if (input_handler == handle_channel_selection) {
        draw_channel_cursor(channel_selection_loc, COL_ACC_0);
    } else {
        draw_channel_cursor(channel_selection_loc, COL_OFF);
    }
    if (input_handler == handle_pattern_selection) {
        draw_pattern_cursor(pattern_selection_loc, COL_ACC_0);
    } else {
        draw_pattern_cursor(pattern_selection_loc, COL_OFF);
    }
    if (input_handler == handle_right_col_selection) {
        draw_right_col_cursor(right_col_selection_loc, COL_ACC_0);
    }
    if (input_handler == handle_param_selection_sq1 ||
            input_handler == handle_param_selection_sq2 ||
            input_handler == handle_param_selection_wave ||
            input_handler == handle_param_selection_noise) {
        draw_params_cursor(param_selection_loc, COL_ACC_0);
        draw_trig_cursor(trig_selection_loc, COL_OFF);
    }
    if (input_handler == handle_param_selection_ch1 ||
            input_handler == handle_param_selection_ch2 ||
            input_handler == handle_param_selection_ch3 ||
            input_handler == handle_param_selection_ch4) {
        draw_params_cursor(param_selection_loc, COL_ACC_0);
    }
    if (input_handler == handle_pattern_chain) {
        draw_pattern_chain_cursor();
    }
}

TriggerNote * get_current_trig(void);

void
draw_piano_notes(void) {
    draw_piano();
    if (input_handler == handle_channel_selection) {
        // Show note on current channel only.
        Pattern *pat = &patterns[current_pattern];
        u8 step = (step_counter - 1) % 16;
        switch (channel_selection_loc) {
            case 0: {
                if (pat->ch1.active && pat->ch1.notes[step].active) {
                    draw_note(pat->ch1.notes[step].note, COL_OFF);
                }
            } break;
            case 1: {
                if (pat->ch2.active && pat->ch2.notes[step].active) {
                    draw_note(pat->ch2.notes[step].note, COL_OFF);
                }
            } break;
            case 2: {
                if (pat->ch3.active && pat->ch3.notes[step].active) {
                    draw_note(pat->ch3.notes[step].note, COL_OFF);
                }
            } break;
        }

    } else if (input_handler == handle_trigger_selection ||
            input_handler == handle_param_selection_sq1 ||
            input_handler == handle_param_selection_sq2 ||
            input_handler == handle_param_selection_wave ||
            input_handler == handle_param_selection_noise) {
        // Show currently selected trigger note.
        TriggerNote *trig = get_current_trig();
        draw_note(trig->note, COL_OFF);
    } else {
        // Show last/current played notes in all channels.
        if (play_status == 1) {
            Pattern *pat = &patterns[current_pattern];
            u8 step = (step_counter - 1) % 16;
            if (pat->ch3.active && pat->ch3.notes[step].active) {
                draw_note(pat->ch3.notes[step].note, COL_OFF);
            }
            if (pat->ch2.active && pat->ch2.notes[step].active) {
                draw_note(pat->ch2.notes[step].note, COL_OFF);
            }
            if (pat->ch1.active && pat->ch1.notes[step].active) {
                draw_note(pat->ch1.notes[step].note, COL_OFF);
            }
        }
    }
}

void
draw_notif_param_edit_prob(u8 prob, u8 x, u8 y, u8 color) {
    txt_drawf_small("PROB/CONDITION: ", x + 2, y + 1, color);
    switch (prob) {
        case PROB_100:         { txt_drawf_small("100", x + 2 + 16 * 4, y + 1, color); txt_drawc_small('%', x + 2 + 19 * 4, y + 1, color); } break;
        case PROB_80:          { txt_drawf_small("80", x + 2 + 16 * 4, y + 1, color);  txt_drawc_small('%', x + 2 + 18 * 4, y + 1, color); } break;
        case PROB_60:          { txt_drawf_small("60", x + 2 + 16 * 4, y + 1, color);  txt_drawc_small('%', x + 2 + 18 * 4, y + 1, color); } break;
        case PROB_40:          { txt_drawf_small("40", x + 2 + 16 * 4, y + 1, color);  txt_drawc_small('%', x + 2 + 18 * 4, y + 1, color); } break;
        case PROB_20:          { txt_drawf_small("20", x + 2 + 16 * 4, y + 1, color);  txt_drawc_small('%', x + 2 + 18 * 4, y + 1, color); } break;
        case PROB_FIRST:       { txt_drawf_small("first bar", x + 2 + 16 * 4, y + 1, color); } break;
        case PROB_NOT_FIRST:   { txt_drawf_small("not first bar", x + 2 + 16 * 4, y + 1, color); } break;
        case PROB_ONE_TWO:     { txt_drawf_small("1st bar out of 2", x + 2 + 16 * 4, y + 1, color); } break;
        case PROB_TWO_TWO:     { txt_drawf_small("2nd bar out of 2", x + 2 + 16 * 4, y + 1, color); } break;
        case PROB_ONE_THREE:   { txt_drawf_small("1st bar out of 3", x + 2 + 16 * 4, y + 1, color); } break;
        case PROB_TWO_THREE:   { txt_drawf_small("2nd bar out of 3", x + 2 + 16 * 4, y + 1, color); } break;
        case PROB_THREE_THREE: { txt_drawf_small("3rd bar out of 3", x + 2 + 16 * 4, y + 1, color); } break;
        case PROB_ONE_FOUR:    { txt_drawf_small("1st bar out of 4", x + 2 + 16 * 4, y + 1, color); } break;
        case PROB_TWO_FOUR:    { txt_drawf_small("2nd bar out of 4", x + 2 + 16 * 4, y + 1, color); } break;
        case PROB_THREE_FOUR:  { txt_drawf_small("3rd bar out of 4", x + 2 + 16 * 4, y + 1, color); } break;
        case PROB_FOUR_FOUR:   { txt_drawf_small("4th bar out of 4", x + 2 + 16 * 4, y + 1, color); } break;
    }
}

void
draw_notif_param_edit_pan(s8 pan, u8 x, u8 y, u8 color) {
    txt_drawf_small("panning: ", x + 2, y + 1, color);
    switch (pan) {
        case -1: { txt_drawf_small("left",   x + 2 + 9 * 4, y + 1, color); } break;
        case 0:  { txt_drawf_small("middle", x + 2 + 9 * 4, y + 1, color); } break;
        case 1:  { txt_drawf_small("right",  x + 2 + 9 * 4, y + 1, color); } break;
    }
}

void
draw_notif_param_env_vol(u8 vol, u8 x, u8 y, u8 color) {
    txt_drawf_small("VOL: ", x + 2, y + 1, color);
    switch (vol) {
        case 0:  { txt_drawf_small("0",   x + 2 + 5 * 4, y + 1, color); txt_drawc_small('%', x + 2 + 6 * 4, y + 1, color); } break;
        case 1:  { txt_drawf_small("6",   x + 2 + 5 * 4, y + 1, color); txt_drawc_small('%', x + 2 + 6 * 4, y + 1, color); } break;
        case 2:  { txt_drawf_small("13",  x + 2 + 5 * 4, y + 1, color); txt_drawc_small('%', x + 2 + 7 * 4, y + 1, color); } break;
        case 3:  { txt_drawf_small("20",  x + 2 + 5 * 4, y + 1, color); txt_drawc_small('%', x + 2 + 7 * 4, y + 1, color); } break;
        case 4:  { txt_drawf_small("26",  x + 2 + 5 * 4, y + 1, color); txt_drawc_small('%', x + 2 + 7 * 4, y + 1, color); } break;
        case 5:  { txt_drawf_small("33",  x + 2 + 5 * 4, y + 1, color); txt_drawc_small('%', x + 2 + 7 * 4, y + 1, color); } break;
        case 6:  { txt_drawf_small("40",  x + 2 + 5 * 4, y + 1, color); txt_drawc_small('%', x + 2 + 7 * 4, y + 1, color); } break;
        case 7:  { txt_drawf_small("46",  x + 2 + 5 * 4, y + 1, color); txt_drawc_small('%', x + 2 + 7 * 4, y + 1, color); } break;
        case 8:  { txt_drawf_small("53",  x + 2 + 5 * 4, y + 1, color); txt_drawc_small('%', x + 2 + 7 * 4, y + 1, color); } break;
        case 9:  { txt_drawf_small("60",  x + 2 + 5 * 4, y + 1, color); txt_drawc_small('%', x + 2 + 7 * 4, y + 1, color); } break;
        case 10: { txt_drawf_small("66",  x + 2 + 5 * 4, y + 1, color); txt_drawc_small('%', x + 2 + 7 * 4, y + 1, color); } break;
        case 11: { txt_drawf_small("73",  x + 2 + 5 * 4, y + 1, color); txt_drawc_small('%', x + 2 + 7 * 4, y + 1, color); } break;
        case 12: { txt_drawf_small("80",  x + 2 + 5 * 4, y + 1, color); txt_drawc_small('%', x + 2 + 7 * 4, y + 1, color); } break;
        case 13: { txt_drawf_small("86",  x + 2 + 5 * 4, y + 1, color); txt_drawc_small('%', x + 2 + 7 * 4, y + 1, color); } break;
        case 14: { txt_drawf_small("93",  x + 2 + 5 * 4, y + 1, color); txt_drawc_small('%', x + 2 + 7 * 4, y + 1, color); } break;
        case 15: { txt_drawf_small("100", x + 2 + 5 * 4, y + 1, color); txt_drawc_small('%', x + 2 + 8 * 4, y + 1, color); } break;
    }
}

void
draw_notif_param_env_time(u8 time, u8 x, u8 y, u8 color) {
    txt_drawf_small("TIME: ", x + 2, y + 1, color);
    switch (time) {
        case 0:  { txt_drawf_small("off",   x + 2 + 6 * 4, y + 1, color); } break;
        case 1:  { txt_drawf_small("150ms", x + 2 + 6 * 4, y + 1, color); } break;
        case 2:  { txt_drawf_small("310ms", x + 2 + 6 * 4, y + 1, color); } break;
        case 3:  { txt_drawf_small("460ms", x + 2 + 6 * 4, y + 1, color); } break;
        case 4:  { txt_drawf_small("620ms", x + 2 + 6 * 4, y + 1, color); } break;
        case 5:  { txt_drawf_small("780ms", x + 2 + 6 * 4, y + 1, color); } break;
        case 6:  { txt_drawf_small("930ms", x + 2 + 6 * 4, y + 1, color); } break;
        case 7:  { txt_drawf_small("1s",    x + 2 + 6 * 4, y + 1, color); } break;
    }
}

void
draw_notif_param_env_dir(u8 dir, u8 x, u8 y, u8 color) {
    txt_drawf_small("DIRECTION: ", x + 2, y + 1, color);
    switch (dir) {
        case 0:  { txt_drawf_small("down", x + 2 + 11 * 4, y + 1, color); } break;
        case 1:  { txt_drawf_small("up",   x + 2 + 11 * 4, y + 1, color); } break;
    }
}

void
draw_notif_bar() {
    u8 x0 = NOTIF_START_X;
    u8 y0 = NOTIF_START_Y - 7;
    u8 x1 = NOTIF_START_X + NOTIF_W;
    u8 y1 = y0 + NOTIF_H;
    u8 color = COL_FG;
    draw_filled_rect(x0, y0, x1, y1, COL_BG);
    draw_rect(x0, y0, x1, y1, color);
    if (notif.time > 0) {
        char msg[32] = {0};
        if (notif.time <= 32) {
            for (s16 i = 0; i < notif.time; i++) {
                msg[i] = notif.msg[i + 32 - notif.time];
            }
        } else {
            for (size_t i = 0; i < 32; i++) {
                msg[i] = notif.msg[i];
            }
        }
        txt_drawf_small(msg, x0 + 2, y0 + 1, color);
    } else {
        Pattern *pat = &patterns[pattern_selection_loc];
        if (input_handler == handle_param_selection_sq1
                || input_handler == handle_param_selection_ch1) {
            ChannelSquareParams *params;
            if (input_handler == handle_param_selection_sq1) {
                params = &pat->ch1.params[trig_selection_loc];
            } else {
                params = &ch1_params;
            }
            switch (param_selection_loc) {
                case 0: {
                    txt_drawf_small("SHAPE: ", x0 + 2, y0 + 1, color);
                    switch (params->duty_cycle) {
                        case 0: { txt_drawf_small("12", x0 + 2 + 7 * 4, y0 + 1, color); } break;
                        case 1: { txt_drawf_small("25", x0 + 2 + 7 * 4, y0 + 1, color); } break;
                        case 2: { txt_drawf_small("50", x0 + 2 + 7 * 4, y0 + 1, color); } break;
                        case 3: { txt_drawf_small("75", x0 + 2 + 7 * 4, y0 + 1, color); } break;
                    }
                    txt_drawc_small('%', x0 + 2 + 9 * 4, y0 + 1, color);
                } break;
                case 1: { draw_notif_param_env_vol(params->env_volume, x0, y0, color); } break;
                case 2: { draw_notif_param_env_time(params->env_time, x0, y0, color); } break;
                case 3: { draw_notif_param_env_dir(params->env_direction, x0, y0, color); } break;
                case 4: { draw_notif_param_edit_prob(params->prob, x0, y0, color); } break;
                case 5: {
                    txt_drawf_small("SWEEP BITS: ", x0 + 2, y0 + 1, color);
                    switch (params->sweep_number) {
                        case 0:  { txt_drawf_small("0", x0 + 2 + 12 * 4, y0 + 1, color); } break;
                        case 1:  { txt_drawf_small("1", x0 + 2 + 12 * 4, y0 + 1, color); } break;
                        case 2:  { txt_drawf_small("2", x0 + 2 + 12 * 4, y0 + 1, color); } break;
                        case 3:  { txt_drawf_small("3", x0 + 2 + 12 * 4, y0 + 1, color); } break;
                        case 4:  { txt_drawf_small("4", x0 + 2 + 12 * 4, y0 + 1, color); } break;
                        case 5:  { txt_drawf_small("5", x0 + 2 + 12 * 4, y0 + 1, color); } break;
                        case 6:  { txt_drawf_small("6", x0 + 2 + 12 * 4, y0 + 1, color); } break;
                        case 7:  { txt_drawf_small("7", x0 + 2 + 12 * 4, y0 + 1, color); } break;
                    }
                } break;
                case 6: {
                    txt_drawf_small("SWEEP TIME: ", x0 + 2, y0 + 1, color);
                    switch (params->sweep_time) {
                        case 0:  { txt_drawf_small("off", x0 + 2 + 12 * 4, y0 + 1, color); } break;
                        case 1:  { txt_drawf_small("7ms", x0 + 2 + 12 * 4, y0 + 1, color); } break;
                        case 2:  { txt_drawf_small("15ms", x0 + 2 + 12 * 4, y0 + 1, color); } break;
                        case 3:  { txt_drawf_small("23ms", x0 + 2 + 12 * 4, y0 + 1, color); } break;
                        case 4:  { txt_drawf_small("31ms", x0 + 2 + 12 * 4, y0 + 1, color); } break;
                        case 5:  { txt_drawf_small("39ms", x0 + 2 + 12 * 4, y0 + 1, color); } break;
                        case 6:  { txt_drawf_small("46ms", x0 + 2 + 12 * 4, y0 + 1, color); } break;
                        case 7:  { txt_drawf_small("54ms", x0 + 2 + 12 * 4, y0 + 1, color); } break;
                    }
                } break;
                case 7: {
                    txt_drawf_small("SWEEP DIRECTION: ", x0 + 2, y0 + 1, color);
                    switch (params->sweep_direction) {
                        case 0:  { txt_drawf_small("up", x0 + 2 + 17 * 4, y0 + 1, color); } break;
                        case 1:  { txt_drawf_small("down", x0 + 2 + 17 * 4, y0 + 1, color); } break;
                    }
                } break;
                case 9: { draw_notif_param_edit_pan(params->pan, x0, y0, color); } break;
            }
            return;
        }

        if (input_handler == handle_param_selection_sq2
                || input_handler == handle_param_selection_ch2) {
            ChannelSquareParams *params;
            if (input_handler == handle_param_selection_sq2) {
                params = &pat->ch2.params[trig_selection_loc];
            } else {
                params = &ch2_params;
            }
            switch (param_selection_loc) {
                case 0: {
                    txt_drawf_small("SHAPE: ", x0 + 2, y0 + 1, color);
                    switch (params->duty_cycle) {
                        case 0: { txt_drawf_small("12", x0 + 2 + 7 * 4, y0 + 1, color); } break;
                        case 1: { txt_drawf_small("25", x0 + 2 + 7 * 4, y0 + 1, color); } break;
                        case 2: { txt_drawf_small("50", x0 + 2 + 7 * 4, y0 + 1, color); } break;
                        case 3: { txt_drawf_small("75", x0 + 2 + 7 * 4, y0 + 1, color); } break;
                    }
                    txt_drawc_small('%', x0 + 2 + 9 * 4, y0 + 1, color);
                } break;
                case 1: { draw_notif_param_env_vol(params->env_volume, x0, y0, color); } break;
                case 2: { draw_notif_param_env_time(params->env_time, x0, y0, color); } break;
                case 3: { draw_notif_param_env_dir(params->env_direction, x0, y0, color); } break;
                case 4: { draw_notif_param_edit_prob(params->prob, x0, y0, color); } break;
                case 9: { draw_notif_param_edit_pan(params->pan, x0, y0, color); } break;
            }
            return;
        }

        if (input_handler == handle_param_selection_wave
                || input_handler == handle_param_selection_ch3) {
            ChannelWaveParams *params;
            if (input_handler == handle_param_selection_wave) {
                params = &pat->ch3.params[trig_selection_loc];
            } else {
                params = &ch3_params;
            }
            switch (param_selection_loc) {
                case 0: {
                    txt_drawf_small("SHAPE A: ", x0 + 2, y0 + 1, color);
                    switch (params->shape_a) {
                        case 0: { txt_drawf_small("sin",    x0 + 2 + 9 * 4, y0 + 1, color); } break;
                        case 1: { txt_drawf_small("saw",    x0 + 2 + 9 * 4, y0 + 1, color); } break;
                        case 2: { txt_drawf_small("square", x0 + 2 + 9 * 4, y0 + 1, color); } break;
                    }
                } break;
                case 1: {
                    txt_drawf_small("TYPE A: ", x0 + 2, y0 + 1, color);
                    switch (params->type_a) {
                        case 0: { txt_drawf_small("a", x0 + 2 + 8 * 4, y0 + 1, color); } break;
                        case 1: { txt_drawf_small("b", x0 + 2 + 8 * 4, y0 + 1, color); } break;
                        case 2: { txt_drawf_small("c", x0 + 2 + 8 * 4, y0 + 1, color); } break;
                        case 3: { txt_drawf_small("d", x0 + 2 + 8 * 4, y0 + 1, color); } break;
                        case 4: { txt_drawf_small("e", x0 + 2 + 8 * 4, y0 + 1, color); } break;
                        case 5: { txt_drawf_small("f", x0 + 2 + 8 * 4, y0 + 1, color); } break;
                        case 6: { txt_drawf_small("g", x0 + 2 + 8 * 4, y0 + 1, color); } break;
                        case 7: { txt_drawf_small("h", x0 + 2 + 8 * 4, y0 + 1, color); } break;
                    }
                } break;
                case 2: {
                    txt_drawf_small("SHAPE B: ", x0 + 2, y0 + 1, color);
                    switch (params->shape_b) {
                        case 0: { txt_drawf_small("sin",    x0 + 2 + 9 * 4, y0 + 1, color); } break;
                        case 1: { txt_drawf_small("saw",    x0 + 2 + 9 * 4, y0 + 1, color); } break;
                        case 2: { txt_drawf_small("square", x0 + 2 + 9 * 4, y0 + 1, color); } break;
                    }
                } break;
                case 3: {
                    txt_drawf_small("TYPE B: ", x0 + 2, y0 + 1, color);
                    switch (params->type_b) {
                        case 0: { txt_drawf_small("a", x0 + 2 + 8 * 4, y0 + 1, color); } break;
                        case 1: { txt_drawf_small("b", x0 + 2 + 8 * 4, y0 + 1, color); } break;
                        case 2: { txt_drawf_small("c", x0 + 2 + 8 * 4, y0 + 1, color); } break;
                        case 3: { txt_drawf_small("d", x0 + 2 + 8 * 4, y0 + 1, color); } break;
                        case 4: { txt_drawf_small("e", x0 + 2 + 8 * 4, y0 + 1, color); } break;
                        case 5: { txt_drawf_small("f", x0 + 2 + 8 * 4, y0 + 1, color); } break;
                        case 6: { txt_drawf_small("g", x0 + 2 + 8 * 4, y0 + 1, color); } break;
                        case 7: { txt_drawf_small("h", x0 + 2 + 8 * 4, y0 + 1, color); } break;
                    }
                } break;
                case 4: { draw_notif_param_edit_prob(params->prob, x0, y0, color); } break;
                case 5: {
                    txt_drawf_small("VOICE: ", x0 + 2, y0 + 1, color);
                    switch (params->wave_mode) {
                        case 0: { txt_drawf_small("A",     x0 + 2 + 7 * 4, y0 + 1, color); } break;
                        case 1: { txt_drawf_small("B",     x0 + 2 + 7 * 4, y0 + 1, color); } break;
                        case 2: { txt_drawf_small("A + B", x0 + 2 + 7 * 4, y0 + 1, color); } break;
                    }
                } break;
                case 6: {
                    txt_drawf_small("VOLUME: ", x0 + 2, y0 + 1, color);
                    switch (params->wave_volume) {
                        case 0: { txt_drawf_small("0",   x0 + 2 + 8 * 4, y0 + 1, color); txt_drawc_small('%', x0 + 2 + 9 * 4, y0 + 1, color); } break;
                        case 1: { txt_drawf_small("25",  x0 + 2 + 8 * 4, y0 + 1, color); txt_drawc_small('%', x0 + 2 + 10 * 4, y0 + 1, color); } break;
                        case 2: { txt_drawf_small("50",  x0 + 2 + 8 * 4, y0 + 1, color); txt_drawc_small('%', x0 + 2 + 10 * 4, y0 + 1, color); } break;
                        case 3: { txt_drawf_small("75",  x0 + 2 + 8 * 4, y0 + 1, color); txt_drawc_small('%', x0 + 2 + 10 * 4, y0 + 1, color); } break;
                        case 4: { txt_drawf_small("100", x0 + 2 + 8 * 4, y0 + 1, color); txt_drawc_small('%', x0 + 2 + 11 * 4, y0 + 1, color); } break;
                    }
                } break;
                case 9: { draw_notif_param_edit_pan(params->pan, x0, y0, color); } break;
            }
            return;
        }

        if (input_handler == handle_param_selection_noise
                || input_handler == handle_param_selection_ch4) {
            ChannelNoiseParams *params;
            if (input_handler == handle_param_selection_noise) {
                params = &pat->ch4.params[trig_selection_loc];
            } else {
                params = &ch4_params;
            }
            switch (param_selection_loc) {
                case 0: {
                    txt_drawf_small("MODE: ", x0 + 2, y0 + 1, color);
                    switch (params->bit_mode) {
                        case 0: { txt_drawf_small("a", x0 + 2 + 6 * 4, y0 + 1, color); } break;
                        case 1: { txt_drawf_small("b", x0 + 2 + 6 * 4, y0 + 1, color); } break;
                    }
                } break;
                case 1: { draw_notif_param_env_vol(params->env_volume, x0, y0, color); } break;
                case 2: { draw_notif_param_env_time(params->env_time, x0, y0, color); } break;
                case 3: { draw_notif_param_env_dir(params->env_direction, x0, y0, color); } break;
                case 4: { draw_notif_param_edit_prob(params->prob, x0, y0, color); } break;
                case 9: { draw_notif_param_edit_pan(params->pan, x0, y0, color); } break;
            }
            return;
        }

        if (settings.help == HELP_ON) {
            if (input_handler == handle_trigger_selection ||
                input_handler == handle_channel_selection) {
                txt_drawf_small("L/R:NOTE  A:PARAMS  B:TOGGLE", x0 + 2, y0 + 1, color);
            } else if (input_handler == handle_param_selection_sq1 ||
                    input_handler == handle_param_selection_sq2 ||
                    input_handler == handle_param_selection_wave ||
                    input_handler == handle_param_selection_noise) {
                txt_drawf_small("L/R:ADJUST  SELECT:COPY", x0 + 2, y0 + 1, color);
            } else if (input_handler == handle_pattern_selection) {
                txt_drawf_small("L/R:CHAIN  A:PARAMS  B:QUEUE", x0 + 2, y0 + 1, color);
            } else if (input_handler == handle_right_col_selection) {
                if (right_col_selection_loc == R_COL_STOP) {
                    txt_drawf_small("B:STOP", x0 + 2, y0 + 1, color);
                } else if (right_col_selection_loc == R_COL_PLAY) {
                    if (play_status == 0) {
                        txt_drawf_small("B:PLAY", x0 + 2, y0 + 1, color);
                    } else {
                        txt_drawf_small("B:PAUSE", x0 + 2, y0 + 1, color);
                    }
                } else if (right_col_selection_loc == R_COL_BPM) {
                    txt_drawf_small("L/R:TEMPO", x0 + 2, y0 + 1, color);
                } else if (right_col_selection_loc == R_COL_SETTINGS) {
                    txt_drawf_small("B:SETTINGS", x0 + 2, y0 + 1, color);
                } else if (right_col_selection_loc == R_COL_BANK_A ||
                        right_col_selection_loc == R_COL_BANK_B ||
                        right_col_selection_loc == R_COL_BANK_C ||
                        right_col_selection_loc == R_COL_BANK_D) {
                    txt_drawf_small("B:SAVE CURRENT BANK AND SWITCH", x0 + 2, y0 + 1, color);
                }
            }
            return;
        }

        if (chain.len != 0) {
            u8 x = x0 + 2;
            u8 y = y0 + 1;
            txt_drawf_small("CHAIN: ", x, y, color);
            x += 4 * 7;
            bool first = true;
            for (size_t i = 0, k = 0; i < MAX_CHAIN; i++) {
                if (chain.active[i] != 1) {
                    continue;
                }
                if (!first) {
                    txt_drawc_small('-', x + 4 * k++, y, COL_FG);
                }
                u8 color = COL_FG;
                if (i == chain.current) {
                    color = COL_ACC_2;
                }
                txt_drawc_small('A' + chain.chain[i], x + 4 * k++, y, color);
                first = false;
            }
        } else {
            if (play_status == 0) {
                txt_drawf_small("STOPPED", x0 + 2, y0 + 1, color);
            } else {
                txt_drawf_small("PLAYING", x0 + 2, y0 + 1, color);
            }
        }
    }
}

void
draw_pattern_chain() {
    clear_parameters();
    for (size_t i = 0; i < 16; i++) {
        u8 color = COL_FG;
        size_t offset_x = PAT_TRIG_OFFSET_X * (i % 8);
        size_t offset_y = i < 8 ? 0 : 0 + PAT_TRIG_OFFSET_Y;
        size_t x0 = PAT_TRIG_START_X + offset_x;
        size_t x1 = PAT_TRIG_START_X + offset_x + PAT_TRIG_W;
        size_t y0 = PAT_TRIG_START_Y + offset_y;
        size_t y1 = PAT_TRIG_START_Y + offset_y + PAT_TRIG_H;
        draw_rect(x0, y0, x1, y1, color);
        if (chain.active[i]) {
            txt_drawc('A' + chain.chain[i], x0 + 4, y0 + 3, color);
        }
        color = COL_OFF;
        if (chain.current == i && chain.len != 0) {
            color = COL_ACC_2;
            if (chain.active[i]) {
                txt_drawc('A' + chain.chain[i], x0 + 4, y0 + 3, color);
            }
            draw_rect(x0, y0, x1, y1, color);
        }
        draw_line(x0 + 5, y1 - 2, x1 - 5, y1 - 2, color);
    }
}
